/* Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * dsndbd.h
 *
 *  Created on: Feb 25, 2010
 *      Author: drewyrr
 */

#ifndef DSNDBD_H_
#define DSNDBD_H_

#include <apr_errno.h>
#include <apr_pools.h>
#include <apu.h>
#include <apr_dbd.h>
#include <apr_file_io.h>
#include <apr_buckets.h>

typedef struct dsndbd_rec {
	apr_pool_t *pool;
	int max_body_size;
	const char *driver_name;
	const char *params;
	const apr_dbd_driver_t *driver;
} dsndbd_rec;

typedef struct dsnconn_rec {
	apr_pool_t *pool;
	dsndbd_rec *dsndbd;
	apr_dbd_t *handle;
	apr_dbd_transaction_t *transaction;
	apr_dbd_prepared_t *select_message;
	apr_dbd_prepared_t *insert_message;
	apr_dbd_prepared_t *select_header;
	apr_dbd_prepared_t *insert_header;
	apr_dbd_prepared_t *select_value;
	apr_dbd_prepared_t *insert_value;
	apr_dbd_prepared_t *select_recipient;
	apr_dbd_prepared_t *insert_recipient;
	apr_dbd_prepared_t *select_recipient_headers;
	apr_dbd_prepared_t *insert_recipient_headers;
	apr_dbd_prepared_t *select_indexed_recipient_headers;
	apr_dbd_prepared_t *insert_indexed_recipient_headers;
	apr_dbd_prepared_t *select_notification;
	apr_dbd_prepared_t *insert_notification;
	apr_dbd_prepared_t *select_notification_headers;
	apr_dbd_prepared_t *insert_notification_headers;
	apr_dbd_prepared_t *select_indexed_notification_headers;
	apr_dbd_prepared_t *insert_indexed_notification_headers;
} dsnconn_rec;

dsndbd_rec *dsndbd_make(apr_pool_t *parent);

apr_status_t dsndbd_close_database(dsnconn_rec *con, dsndbd_rec *conf, apr_pool_t *pool);

apr_status_t dsndbd_open_database(dsnconn_rec **con, dsndbd_rec *conf, apr_pool_t *pool);

apr_status_t dsndbd_update_message(apr_pool_t *pool, dsnconn_rec *dsncon,
		const char *recipient, apr_table_t *headers, apr_uint64_t *key);

apr_status_t dsndbd_update_notification(apr_pool_t *pool, dsnconn_rec *dsncon,
		const char *message_id, apr_uint64_t recipient_key, apr_table_t *headers,
		apr_table_t *headers_body, apr_bucket_brigade *body);

#endif /* DSNDBD_H_ */
