/* Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * mimeparse.h
 *
 *  Created on: Feb 10, 2010
 *      Author: drewyrr
 */

#ifndef MIMEPARSE_H_
#define MIMEPARSE_H_

#include <apr_pools.h>
#include <apr_file_io.h>
#include <apr_buckets.h>

#define MESSAGE_NORMAL 0
#define MESSAGE_LAST 1

enum {
    FIND_START = 0, FIND_FROM, FIND_HEADERS, FIND_SEPARATOR, FIND_BOUNDARY
} phase_t;

typedef struct mimeparse_rec mimeparse_rec;
typedef struct mime_rec mime_rec;

struct mimeparse_rec {
    apr_pool_t *pool;
    apr_file_t *err;
    apr_file_t *out;
    apr_file_t *in;
    apr_bucket_alloc_t *alloc;
    mime_rec *mime;
    apr_status_t (*process_line)(mimeparse_rec *, mime_rec *, const char *,
            apr_size_t);
    apr_status_t (*process_message)(mimeparse_rec *, mime_rec *, int);
};

struct mime_rec {
    apr_pool_t *pool;
    mime_rec *first;
    mime_rec *last;
    mime_rec *parent;
    mime_rec *next;
    int phase;
    int level;
    const char *separator;
    apr_bucket_brigade *line;
    apr_table_t *headers;
    const char *boundary;
    int boundary_length;
    apr_bucket_brigade *body;
};

mimeparse_rec *mime_parse_make(apr_pool_t *parent);

mime_rec *mime_make(apr_pool_t *ppool, mimeparse_rec *conf, mime_rec *parent);

void mime_reset(mimeparse_rec *conf, mime_rec *mime);

apr_status_t mime_process_line(mimeparse_rec *conf, mime_rec *mime, const char *str,
        apr_size_t len);

apr_status_t mime_last_line(mimeparse_rec *conf, mime_rec *mime);

apr_status_t mime_parse_header(mimeparse_rec *conf, apr_bucket_brigade *bb,
		apr_table_t *headers);

apr_status_t mime_parse_header_value(mimeparse_rec *conf, const char *raw,
		char **value, apr_table_t *params);

apr_status_t mime_parse_headers(mimeparse_rec *conf, apr_bucket_brigade *bb,
		apr_table_t *headers);

const char *verp_parse(apr_pool_t *pool, const char *verp, const char *sep);

#endif /* MIMEPARSE_H_ */
